/** @file sp/spKanji.h
 */

#ifndef __SPKANJI_H
#define __SPKANJI_H

#ifdef __cplusplus
extern "C" {
#endif

/** @~english @defgroup spKanjiGroup Kanji Handling */

/** @addtogroup spKanjiGroup
 *  @{ */  /*----@addtogroup spKanjiGroup----*/

typedef enum {
    SP_KANJI_CODE_UNKNOWN = -1,
    SP_KANJI_CODE_JIS = 0,
    SP_KANJI_CODE_EUC = 1,
    SP_KANJI_CODE_SJIS = 2,
    SP_KANJI_CODE_ZENKAKU_SJIS = 3,
    SP_KANJI_CODE_ZENKAKU_JIS = 4,
    SP_KANJI_CODE_ZENKAKU_EUC = 5,
    SP_KANJI_CODE_UTF8 = 10,
} spKanjiCode;

#define SP_KANJI_CODE_CURRENT SP_KANJI_CODE_UNKNOWN

/** @} */  /*----@addtogroup spKanjiGroup----*/

#if defined(_WIN32) || defined(MACOS)
#define SP_KANJI_CODE_DEFAULT SP_KANJI_CODE_SJIS
#else
#define SP_KANJI_CODE_DEFAULT SP_KANJI_CODE_EUC
#endif
#if (defined(_WIN32) && !defined(__CYGWIN32__)) || defined(MACOS)
#define SP_KANJI_CODE_INTERNAL SP_KANJI_CODE_SJIS
#else
#define SP_KANJI_CODE_INTERNAL SP_KANJI_CODE_EUC
#endif

#if defined(MACOS)
#pragma import on
#endif

/** @addtogroup spKanjiGroup
 *  @{ */  /*----@addtogroup spKanjiGroup----*/

/** @~english Sets options for nkf engine. */
extern int spSetKanjiOptions(int n, const char **options);
/** @~english Displays the usage of nkf engine. */
extern void spKanjiUsage(const char *name);
/** @~english Sets the flag to determine if MIME is decoded. */
extern void spDecodeMime(int flag);
/** @~english Sets the default Kanji code. */
extern void spSetDefaultKanjiCode(spKanjiCode code);
/** @~english Sets the Kanji codes of input and output. */
extern void spSetKanjiCode(spKanjiCode icode, spKanjiCode ocode);
/** @~english Gets the current Kanji code. */
extern spKanjiCode spGetKanjiCode(void);
/** @~english Gets the label of a Kanji code. */
extern const char *spGetKanjiCodeLabel(spKanjiCode code);
/** @~english Gets the encoding string associated with a Kanji code such as "US", "SJIS" and "UTF-8". */
extern const char *spGetKanjiCodeEncoding(spKanjiCode code);
/** @~english Gets the Kanji code for a locale. */
extern spKanjiCode spGetLocaleKanjiCode(const char *lang);

/* converted buffer always terminated with null */
/** @~english Converts the Kanji code of the input string. */
extern spKanjiCode spConvertKanji(const unsigned char *lin, unsigned char *lout, int size);
/** @~english Converts the Kanji code of the input string. */
extern spKanjiCode spConvertKanjiCode(const unsigned char *lin, unsigned char *lout,
				      int size, spKanjiCode icode, spKanjiCode ocode);
/** @~english Converts the Kanji code of the input string associated with the current locale. */
extern spKanjiCode spConvertKanjiFromLocaleCode(unsigned char *iobuf, int size, spKanjiCode ocode);
/** @~english Converts the Kanji code of the input string into the Kanji code associated with the current locale. */
extern spKanjiCode spConvertKanjiToLocaleCode(unsigned char *iobuf, int size, spKanjiCode icode);
    
/** @} */  /*----@addtogroup spKanjiGroup----*/

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPKANJI_H */
