/** @file sp/spLibrary.h
 */

#ifndef __SPLIBRARY_H
#define __SPLIBRARY_H

#include <sp/spDefs.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @~english @defgroup spLibraryGroup Dynamic Library Loading */

/** @addtogroup spLibraryGroup
 *  @{ */  /*----@addtogroup spLibraryGroup----*/

#if defined(MACOS)
#pragma import on
#endif

/** @~english Opens the dynamic library associated with \p filename. The behavior of multiple function calls for the same library depends on an environment.
 *  On some environments the function calls map the library to multiple address spaces, on others the calls map the library to one address space.
 *  Therefore, you have to write code independent from this behavior. Moreover, the behavior without directory path depends on an environment.
 *  @param[in] filename The null-terminated name of the file associated with the dynamic library.
 *  @return The handle of the library.
 *  @sa spCloseLibrary() */
extern void *spOpenLibrary(const char *filename);
/** @~english Gets the address of the function included in the dynamic library.
 *  @param[in] handle The handle of the dynamic library.
 *  @param[in] symbol The null-terminated name of the function. For example, \p symbol is "func" if the function is func.
 *  @return The address of the function. This address can be used as the pointer to the function.
 *  @sa spOpenLibrary() */
extern void *spGetSymbolAddress(void *handle, const char *symbol);
/** @~english Closes the dynamic library.
 *  @param[in] handle The handle of the dynamic library.
 *  @sa spOpenLibrary() */
extern void spCloseLibrary(void *handle);

#if defined(MACOS)
#pragma import off
#endif

/** @} */  /*----@addtogroup spLibraryGroup----*/

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPLIBRARY_H */
