/** @file sp/spMain.h
 */

#ifndef __SPMAIN_H
#define __SPMAIN_H

#if defined(__cplusplus) && !defined(SP_DISABLE_ABBREVIATION)
#define SP_DISABLE_ABBREVIATION 1
#define SP_ENABLE_ABBREVIATION_LATER 1
#endif
    
#include <sp/spBase.h>

#if defined(SP_ENABLE_ABBREVIATION_LATER)
#undef SP_DISABLE_ABBREVIATION 
#undef SP_ENABLE_ABBREVIATION_LATER    
#endif

#ifdef __cplusplus
extern "C" {
#endif
    
#if defined(MACOS)

#pragma import on
int spMain(int argc, char *argv[]);
#pragma import off

int main(int argc, char *argv[])
{
#if !(defined(MACOSX64) || defined(IPHONE)) && !TARGET_API_MAC_CARBON
    MaxApplZone();
#endif
    return spMain(argc, argv);
}

#elif defined(ANDROID)
#include <jni.h>
#include <android_native_app_glue.h>
#include <android/log.h>

#define  SP_LOG_TAG    "spMain.h"
#define  SP_LOGI(...)  __android_log_print(ANDROID_LOG_INFO,SP_LOG_TAG,__VA_ARGS__)
    
extern void spUpdateDirectoryAndroid(JNIEnv *env, jobject clazzobj);
    
int spMain(int argc, char *argv[]);

static void spEngineHandleCmd(struct android_app* app, int32_t cmd) {
}

static int32_t spEngineHandleInput(struct android_app* app, AInputEvent* event) {
    if (AInputEvent_getType(event) == AINPUT_EVENT_TYPE_KEY) {
        SP_LOGI("Key event: action=%d keyCode=%d metaState=0x%x",
                AKeyEvent_getAction(event),
                AKeyEvent_getKeyCode(event),
                AKeyEvent_getMetaState(event));
    } else if (AInputEvent_getType(event) == AINPUT_EVENT_TYPE_MOTION) {
        SP_LOGI("Motion event: (%f, %f)",
                AMotionEvent_getX(event, 0), AMotionEvent_getY(event, 0));
    }
    return 0;
}

void android_main(struct android_app* state) {
    int ident;
    int events;
    JavaVM *vm;
    JNIEnv *env;
    struct android_poll_source* source;
    char *argv[1] = {"main"};
    
    /*app_dummy();*/
    state->onAppCmd = spEngineHandleCmd;
    state->onInputEvent = spEngineHandleInput;

    vm = state->activity->vm;
    (*vm)->AttachCurrentThread(vm, &env, NULL);
    spUpdateDirectoryAndroid(env, state->activity->clazz);
    (*vm)->DetachCurrentThread(vm);
    
    spMain(1, argv);

    while ((ident = ALooper_pollAll(-1, NULL, &events, (void**)&source)) >= 0) {
	// Process this event.
	if (source != NULL) {
	    source->process(state, source);
	}

	// Check if we are exiting.
	if (state->destroyRequested != 0) {
	    SP_LOGI("Engine thread destroy requested!");
	    return;
	}
    }
}
#elif defined(_WIN32) && !defined(__CYGWIN32__)
int spMain(int argc, char *argv[]);

int main(int argc, char *argv[])
{
#if defined(_DEBUG) && defined(_CRTDBG_MAP_ALLOC)
    _CrtSetDbgFlag (_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);
#endif
    spsetstdio(stdin, stdout, stderr);
    return spMain(argc, argv);
}
#else
#define spMain main
#endif
    
#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPMAIN_H */
