/*
 *	spPluginP.h
 */

#ifndef __SPPLUGINP_H
#define __SPPLUGINP_H

#include <sp/spPlugin.h>
#include <sp/spThread.h>

#ifdef __cplusplus
extern "C" {
#endif

#define SP_PLUGIN_DEFAULT_BUFFER_SIZE 8192

#define SP_PLUGIN_DEFAULT_THREAD_IDS_SIZE 8
    
#if defined(_WIN32)
#if defined(__CYGWIN32__)
#define spPluginExport
#define SP_PLUGIN_DEFAULT_SEARCH_PATH "~/sp_plugins;~/.sp_plugins;c:/Program Files/sp/plugins;c:/usr/lib/sp/plugins"
#else
#ifdef __cplusplus
#define spPluginExport extern "C" __declspec(dllexport)
#else
#define spPluginExport __declspec(dllexport)
#endif
#define SP_PLUGIN_DEFAULT_SEARCH_PATH "~\\sp_plugins;~\\.sp_plugins;c:\\Program Files\\sp\\plugins;c:\\usr\\lib\\sp\\plugins"
#endif
#define SP_PLUGIN_SUFFIX ".dll"

#elif defined(MACOS)
    
#if defined(IPHONE)
#define spPluginExport static
#else
#define spPluginExport
#endif

#if defined(MACOSX)
#define SP_PLUGIN_DEFAULT_SEARCH_PATH "./sp_plugins;./.sp_plugins"
#define SP_PLUGIN_SUFFIX ".bundle"
#define SP_PLUGIN_ANOTHER_SUFFIX ".dylib"
#else
#define SP_PLUGIN_DEFAULT_SEARCH_PATH ":sp_plugins;:.sp_plugins"
#define SP_PLUGIN_SUFFIX ".dll"
#endif

#else
#define spPluginExport

#if defined(ANDROID)
#define SP_PLUGIN_DEFAULT_SEARCH_PATH ""
#else
#if defined(__x86_64__)
#define SP_PLUGIN_DEFAULT_SEARCH_PATH ".:~/.sp_plugins:/usr/lib64/sp/plugins:/usr/lib/x86_64-linux-gnu/sp/plugins:/usr/lib/sp/plugins:/usr/local/lib64/sp/plugins:/usr/local/lib/sp/plugins"
#else
#define SP_PLUGIN_DEFAULT_SEARCH_PATH ".:~/.sp_plugins:/usr/lib/i386-linux-gnu/sp/plugins:/usr/lib/sp/plugins:/usr/local/lib/sp/plugins"
#endif
#endif
#define SP_PLUGIN_SUFFIX ".so"
#endif

#if defined(IPHONE)
#define spUniquePluginExport 
#else
#define spUniquePluginExport spPluginExport
#endif

#define SP_GET_PLUGIN_REC_FUNC "spGetPluginRec"
#define SP_PLUGIN_SYSTEM_VERSION_FUNC "spPluginSystemVersion"
#define SP_PLUGIN_SUPPORTED_VERSION_FUNC "spPluginSupportedVersion"
#define SP_PLUGIN_PATH_ENV "SP_PLUGIN_PATH"

#define SP_PLUGIN_HOST_CR_NONE (-1)
#define SP_PLUGIN_HOST_CR_ERROR 0
#define SP_PLUGIN_HOST_CR_DIALOG_OK 10
#define SP_PLUGIN_HOST_CR_DIALOG_CANCEL 11
#define SP_PLUGIN_HOST_CR_USER 256
typedef int spPluginHostCallbackReason;

typedef spBool (*spPluginHostCallback)(spPluginRec *rec, spPluginHostCallbackReason reason, void *callback_data, void *user_data);

typedef struct _spPluginHostData *spPluginHostData;
    
struct _spPluginRec
{
    void *host_data;		/* host data */
    void *user_data;		/* user data */
    
    spPluginType type;
    const char *plugin_id;
    long version_id;
    int priority;
    unsigned long caps;
    spBool (*init)(const char *lang);
    spBool (*free)(void);
    const char *description;		/* short description */
    const char *information;		/* shown in about dialog box. can include '\n' */
    
    void *(*init_instance)(const char *lang);		/* must return non-NULL */
    spBool (*free_instance)(void *instance);
    spOptions (*init_options)(void *instance, const char *lang); 	/* Don't use pointers to static or global variables for `value'. Use pointers to the variables in an instance. */
    spBool (*free_options)(void *instance, spOptions options);
};

typedef struct _spPluginHostInstanceList *spPluginHostInstanceList;
struct _spPluginHostInstanceList
{
    void *instance;
    spThreadId thread_id;

    spPluginHostInstanceList prev;
    spPluginHostInstanceList next;
};
    
struct _spPluginHost
{
    char *name;			/* plugin file name */
    long version_id;
    long ref_count;
    void *mutex;
    void *data;
    long num_instance;

    char *buffer;
    long buffer_size;

    /* version 1.5+ */
    spPluginHostCallback callback_func;
    void *callback_user_data;

    /* version 1.6+ */
    spPluginHostInstanceList instance_list;

    /* version 1.7+ */
    void *plugin_specific_data;
};

struct _spPlugin
{
    spPluginHost *host;
    spPluginRec *rec;
    
    /* items for instance */
    void *handle;		/* plugin handle */
    void *instance;		/* plugin instance */
};

#if defined(MACOS)
#pragma import on
#endif

extern spPluginHostData spGetPluginHostData(void);
extern spPluginHostData spGetPluginHostDataFromHost(spPluginHost *host);

extern spPluginRec *spSearchPluginRecList(const char *name, spBool static_only);
extern const char *spGetPluginRecListName(int index, spBool static_only);
extern spPlugin *spLoadPluginFromHostData(spPluginHostData host_data, const char *plugin_name);
extern spPlugin *spLoadPluginFromHostMemory(spPluginHost *host, const char *plugin_name);
extern spBool spFreePluginRecHost(spPluginRec *rec);

extern spBool spLockPluginMutex(spPlugin *plugin);
extern spBool spUnlockPluginMutex(spPlugin *plugin);

extern spBool spEqPluginTypeMain(spPluginType x_type, spPluginType y_type);

extern spBool spInitPluginBuffer(spPlugin *plugin, long buffer_size);

/* function for spLibs internal */
extern long spGetPluginRefCount(spPlugin *plugin); /* -1 returned: error */

extern const char *spSearchPluginFileFromHostData(spPluginHostData host_data, int index);
extern char *xspSearchPluginFileFromHostData(spPluginHostData host_data, int index);
    
extern spBool spIsPluginHostCallbackSet(spPlugin *plugin);
extern spBool spSetPluginHostCallback(spPlugin *plugin, spPluginHostCallback callback_func, void *user_data); /* for host */
extern spBool spEmitPluginHostCallback(spPluginRec *rec, spPluginHostCallbackReason reason, void *callback_data); /* for plugin */

extern spBool spAllocPluginSpecificDataToHost(spPluginHost *host, int size);
extern spBool spAllocPluginSpecificData(spPlugin *plugin, int size);
extern void *spGetPluginSpecificDataFromHost(spPluginHost *host);
extern void *spGetPluginSpecificData(spPlugin *plugin);

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPPLUGINP_H */
